C   02/11/89 011272234  MEMBER NAME  ZWPROD   (ZWMS.S)      F77
C.----------------------------------------------------------------------
C.
C.                       MAIN PROGRAM : ZWPRODMS
C.
C.    Written by : R. Hamberg, T. Matsuura and W.L. van Neerven
C.
C-----------------------------------------------------------------------
C.
C. R. Hamberg, W.L. van Neerven and T. Matsuura, Nucl. Phys. B359 (1991) 343.
C.
C-----------------------------------------------------------------------
C
C     REVISED VERSION MADE IN AUGUST 2002
C.
C.    This program calculates the Z and W production rates using all
C.    the corrections that are known at this moment.
C.
C.    List of known corrections in the MSbar scheme :
C.
C.    1st order : q qb  +  q g, both have been completely calculated
C.
C.    2nd order : all subprocesses ( q qb, q q, qb qb, q g and gg)
C.                are included
C.
C.    This program calculates the Z, W(+), W(-) or W(+)+W(-) production
C.    rates for pp and ppb colliders at a specified C.M. energy.
C
C     The main control of this program is handled by the subroutine
C.    KEUZE. By setting the appropriate flags and choosing the right
C.    parameters one can obtain the Z/W production rate in the desired
C.    approximation.
C.
C.    Before running the program one should check whether the subroutine
C.    INIT initializes the parameters to the desired values ( e.g. the
C.    masses of the vector bosons are initialized ).
C.
C.    The user is advised to study at least the subroutines KEUZE,
C.    DSNEDE and INIT in some detail.
C.
C.
C.    When using this program one has to provide sets of parton distri-
C.    bution functions in the subroutine STRUCT, which should be of the
C.    following form :
C.
C.    CALL STRUCT(X,SCALE,MODE,UV,DV,USEA,DSEA,STR,CHM,BOT,GLU)
C.
C.             Here : SCALE = scale in GeV
C.                    MODE  = specifies the parton distribution function
C.                    UV    : up valence quark
C.                    DV    : down valence quark
C.                    USEA  : up sea quark
C.                    DSEA  : down sea quark 
C.                    STR   : strange quark
C.                    CHM   : charm quark
C.                    BOT   : bottom quark
C.                    GLU   : gluon
C.
C.
C.             N.B.: STRUCT returns X * parton distribution function.
C.                   One should check whether :
C.                                1
C.                               /
C.                              | UV(x)/x dx= 2
C.                             /
C.                             0
C.
C.    As the theoretical calculations were performed in the MSbar
C.    scheme, one should preferably use parton distribution functions
C.    determined in the same scheme.
C.
C.
C.    Many of the subroutines and functions may be altered by the user,
C.    but we recommend not to change the following functions :
C.
C.       the functions C*** ( CQB1 - CQQCRF ) containing the Drell-Yan
C.       correction terms
C.       the functions PD** containing the structure functions.
C.
C.
C.----------------------------------------------------------------------
      subroutine wzrateinit

      IMPLICIT REAL*8 (A-H,O-Z)
C.
      DIMENSION TERM(20)
      REAL*8 NONSIN
      REAL*8 MZ,MW,NF
      REAL*8 LAMBDA
      INTEGER BEAM,TARGET
C.
C.    initialization of parameters
CJS*****************************************************
	COMMON/MASS/MZ,MW
	COMMON/WIDTH/GAMZ,GAMW
	COMMON/ANGLE/SINW,COSW,SINC,COSC
	COMMON/CONST/GF,CONVER,ALPHA
	COMMON/KIESPD/MODE
	COMMON/CASI/CF,CA,NF
        COMMON/ORDE/NORDER
        COMMON/KOPPAR/LAMBDA,KOPAPP
        COMMON/SET/NZP,NWM,NWB,NCONST,NFRS
        COMMON/SCALES/FACSCA,RENSCA
        COMMON/COLLISION/BEAM,TARGET
      COMMON/ZORW/ZPROD

      COMMON/WMORWP/WMINUS,WBOTH
      LOGICAL ZPROD,WMINUS,WBOTH
      common /forwzrates/ nfl

C.
C.    initialization of parameters see also iniphz.f
C.
	OPEN (UNIT=11, FILE='zwms.dat', STATUS='OLD')
	REWIND 11

!	READ (11,*) CMENER
!        READ (11,*) BEAM
!        READ (11,*) TARGET
! Set the PDF set in the main program 
!	READ (11,*) MODE
	READ (11,*) MZ
	READ (11,*) MW
	READ (11,*) GAMZ
	READ (11,*) GAMW
        READ (11,*) SINW
c	READ (11,*) SINC
c        READ (11,*) COSC
c	READ (11,*) NFL
	READ (11,*) GF
	READ (11,*) ALPHA
!        READ (11,*) NORDER
!	READ (11,*) KOPAPP
!	READ (11,*) NZP
!        READ (11,*) NWB
!	READ (11,*) NWM
        READ (11,*) NCONST
        READ (11,*) NFRS
        READ (11,*) FACSCA
        READ (11,*) RENSCA

C       PRINT '(''0 PROGRAM ZWPRODMS:	'')'
C       PRINT '(''0 CM-ENERGY (GEV) = '',D20.7)',CMENER
C       PRINT '(''0 BEAM            = '',I2)',BEAM
C       PRINT '(''0 TARGET          = '',I2)',TARGET
C       PRINT '(''0 MODE            = '',I2)',MODE
C       PRINT '(''0 MZ (GEV)        = '',D20.7)',MZ
C       PRINT '(''0 MW (GEV)        = '',D20.7)',MW
C       PRINT '(''0 GAMZ (GEV)      = '',D20.7)',GAMZ
C       PRINT '(''0 GAMW (GEV)      = '',D20.7)',GAMW
C       PRINT '(''0 SINW            = '',D20.7)',SINW
C       PRINT '(''0 SINC            = '',D20.7)',SINC
C       PRINT '(''0 COSC            = '',D20.7)',COSC
C       PRINT '(''0 NF              = '',I2)',NFL
C       PRINT '(''0 GF              = '',D20.7)',GF
C       PRINT '(''0 ALPHA           = '',D20.7)',ALPHA
C       PRINT '(''0 NORDER          = '',I2)',NORDER
C       PRINT '(''0 KOPAPP          = '',I2)',KOPAPP
C       PRINT '(''0 NZP             = '',I2)',NZP
C       PRINT '(''0 NWB             = '',I2)',NWB
C       PRINT '(''0 NWM             = '',I2)',NWM
C       PRINT '(''0 NCONST          = '',I2)',NCONST
C       PRINT '(''0 NFRS            = '',I2)',NFRS
C       PRINT '(''0 FACSCA          = '',D20.7)',FACSCA
C       PRINT '(''0 RENSCA          = '',D20.7)',RENSCA

       NF=DFLOAT(NFL)
CJS    
       SS=CMENER
CJS ADD DUMB VALUE FOR COMMON BLOCK
       LAMBDA=0.1D0
C.
C.    Weinberg angle
C.    SINW = ( sin(theta_W) )**2 = 0.2311
C.
CJS	SINW=0.2311D0
	COSW=1.0D0 - SINW
C.
C.    Cabibbo mixing angle
C.    SINC = ( sin(theta_C) )**2 = 0.048
C.
CJS(DATA FILE)	 SINC=0.048D0

CJS	         COSC=0.948D0

C.    Colour factors
C.    CF=(N**2-1)/(2*N), CA=N, N=#colours
C.    We use N=3
C.
	CF=4.0D0/3.0D0
	CA=3.0D0
C.
C.    NF=#flavours
C.    NF is the number of flavours in the final state, in the loop
C.    diagrams and in the formula for the running coupling constant.
C.
CJS(DATA FILE)	 NF=4.0D0
C.
C.    Fermi coupling constant
C.    GF=1.16639*10**-5 GeV**-2
C.
CJS(DATA FILE)	 GF=1.16639D-5
C.
C.    Conversion factor for GeV**-2 ---> nbarn
C.    1 GEV**-2 = 389379.29 NBARN
C.
	CONVER=389379.29D0
CJS.	fine structure constant
C.
C.	ALPHA=1.0/137.0359895D0
C
CJS(DATA FILE)	 ALPHA=0.7297353D-2
C
CJS**************************************************************
C.
      CALL INIT
C.
C.    KEUZE sets flags and initializes parameters
C.    It should be called by : KEUZE( c.m. energy in GeV )
C.    
C      SS = 630.D0
CJS      SS=1800.D0
      CALL KEUZE(SS)
C.
C.    DSNEDE calculates the various contribution to the total x-section
C.
!  Do it in the main program
!      CALL DSNEDE(TERM)
C.
C.    print the input parameters
C.
      CALL HEAD
      IF(NORDER.EQ.2) THEN
      ZEROTH=TERM(1)
      FIRST=TERM(2)+TERM(3)+TERM(4)
      QBTOT1=TERM(2)+TERM(3)
      SECOND=TERM(5)+TERM(6)+TERM(7)+TERM(8)+TERM(9)+TERM(10)
      NONSIN=TERM(5)+TERM(6)
      QBTOT2=NONSIN+TERM(10)
!      WRITE(6,10) ZEROTH
!      WRITE(6,20) TERM(2),TERM(3),QBTOT1,TERM(4),FIRST,ZEROTH+FIRST
!      WRITE(6,30) TERM(5),TERM(6),NONSIN,TERM(10),QBTOT2,TERM(7),
!     .            TERM(8),TERM(9),SECOND,ZEROTH+FIRST+SECOND
      ELSEIF(NORDER.EQ.1) THEN
      ZEROTH=TERM(1)
      FIRST=TERM(2)+TERM(3)+TERM(4)
      QBTOT1=TERM(2)+TERM(3)
!      WRITE(6,10) ZEROTH
!      WRITE(6,20) TERM(2),TERM(3),QBTOT1,TERM(4),FIRST,ZEROTH+FIRST
      ELSEIF(NORDER.EQ.0) THEN
      ZEROTH=TERM(1)
!      WRITE(6,10) ZEROTH
      ENDIF


10    FORMAT(' The results are ( in nbarn )    : ',/
     .      ,' Born contribution               : ',/
     .      ,' 0th order, q qb                 : ',1PE13.5,/)
20    FORMAT(' 1st order contributions         : ',/
     .      ,' q qb, singular                  : ',1PE13.5,/
     .      ,' q qb, regular                   : ',1PE13.5,/
     .      ,' q qb, total                     : ',1PE13.5,/
     .      ,' q g                             : ',1PE13.5,/
     .      ,' total 1st order contribution    : ',1PE13.5,/
     .      ,' Cross-section up to 1st order   : ',1PE13.5,/)
30    FORMAT(' 2nd order contributions         : ',/
     .      ,' q qb, singular                  : ',1PE13.5,/
     .      ,' q qb, regular                   : ',1PE13.5,/
     .      ,' q qb, nonsinglet                : ',1PE13.5,/
     .      ,' q qb, singlet                   : ',1PE13.5,/
     .      ,' q qb, total                     : ',1PE13.5,/
     .      ,' qg                              : ',1PE13.5,/
     .      ,' g g,                            : ',1PE13.5,/
     .      ,' q q + qb qb                     : ',1PE13.5,/
     .      ,' total 2nd order contribution    : ',1PE13.5,/
     .      ,' Cross-section up to 2nd order   : ',1PE13.5,/)
C.
      return
      END

      SUBROUTINE KEUZE(SQRTS)
C.----------------------------------------------------------------------
C.
C.    This subroutine sets various flags and initializes parameters.
C.
C.    Subroutine call : KEUZE( c.m. energy in GeV )
C.
C.    Flags that are set by KEUZE :
C.
C.    ZPROD  : determines whether Z or W production is calculated
C.             Z production for ZPROD=.TRUE.
C.    WMINUS : determines in case of W production whether W(+) or W(-)
C.             is produced.
C.             W(-) for WMINUS=.TRUE.
C.    WBOTH  : if .TRUE., then both W(+) and W(-) are produced.
C.
C.    Choose a beam particle:
C.    BEAM = 1 : proton
C.           2 : anti-proton
C.           3 : neutron
C.
C.    Choose a target particle:
C.    TARGET = 1 : proton
C.             2 : neutron
C.
C.    FERMI  : we have two different formulae for the overall
C.             normalization of the Drell-Yan process, one where the
C.             fine structure constant is used and another where the
C.             Fermi coupling constant is used.
C.             FERMI=.TRUE. for the formula with the Fermi constant.
C.    NATFRS : if NATFRS=.TRUE., then the 'natural' scales are used for
C.             the running coupling constant and the parton distribution
C.             functions. ( 'natural' scale = vector boson mass )
C.             Otherwise one can define one's own massfactorization and
C.             renormalization scale.
C.
C.    parameters initialized by KEUZE
C.
C.    S      : ( c.m. energy in GeV )**2
C.    TAU    : scaling variable = ( vector boson mass )**2/S
C.
C.    choosing the relevant scales :
C.    FACSCA : massfactorization scale ( used in parton distr. funct. ).
C.             For NATFRS=.TRUE. this equals the vector boson mass,
C.             otherwise one is free to choose any other value which is
C.             not to far from the 'natural' scale.
C.    RENSCA : renormalization scale ( used in the coupl. constant ).
C.             Same comments as for FACSCA.
C.
C.    MODE   : chooses a parton distribution function set.
C.             One may add other parton distribution functions.
C.             Such a parton distribution function should be put into
C.             the subroutine STRUCT, which is of the following form:
C.
C.    CALL STRUCT(X,SCALE,MODE,UV,DV,USEA,DSEA,STR,CHM,BOT,GLU)
C.
C.             Here : SCALE = scale in GeV
C.                    MODE  = specifies the parton distribution function
C.                    UV    : up valence quark
C.                    DV    : down valence quark
C.                    USEA  : up sea quark 
C.                    DSEA  : down sea quark 
C.                    STR   : strange quark
C.                    CHM   : charm quark
C.                    BOT   : bottom quark
C.                    GLU   : gluon
C.
C.
C.             N.B.: STRUCT returns X * parton distribution function.
C.
C.    parameters for the running coupling constant :
C.    LAMBDA : QCD scale, should be matched with the scale used in the
C.             parton distribution function.
C.    KOPAPP : determines whether the leading or the next-to-leading
C.             order formula for the running coupling constant is used.
C.             KOPAPP=1 : leading order, KOPAPP=2 : next-to-leading order
C.
C.    choosing the approximation for the calculation :
C.    NORDER : order of the q qb, non-singlet contribution ( 0, 1 or 2 )
C.    REG1   : if .TRUE., 1st order, q qb, hard is included
C.    QUGL1  : if .TRUE., 1st order, q g is included
C.    REG2   : if .TRUE., 2nd order, q qb, is included
C.    QUGL2  : if .TRUE., 2nd order, q g is included
C.    GLGL   : if .TRUE., g g is included
C.    QUQU   : if .TRUE., q q and qb qb are included
C.    QUQUBS : if .TRUE., q qb, singlet is included
C.    EXPECT : in the MSbar scheme : dummy flag
C.    REMARK : for a complete calculation put NORDER=2 and set all the
C.             flags equal to .TRUE.
C.
C.    VEGAS parameters ( used in the subroutine INTVEG ) :
C.    NPRINT : controls the output by VEGAS ( Monte Carlo integration
C.             routine ). For NPRINT=0 all the output is suppresses,
C.             for NPRINT=1 information is given about the numerical
C.             integrations.
C.    ITi    : (i=1,2), number of iterations in the ith VEGAS call
C.    Ni     : (i=1,2), number of Monte Carlo points in the ith VEGAS
C.             call.
C.
C.----------------------------------------------------------------------
C.
      IMPLICIT REAL*8 (A-H,O-Z)
      REAL*8 MZ,MW
      REAL*8 LAMBDA
      INTEGER BEAM,TARGET
C.
      COMMON/ZORW/ZPROD
      LOGICAL ZPROD
      COMMON/WMORWP/WMINUS,WBOTH
      LOGICAL WMINUS,WBOTH
      COMMON/ENERGY/S,TAU
      COMMON/MASS/MZ,MW
      COMMON/NORM/FERMI
      LOGICAL FERMI
      COMMON/KIESPD/MODE
      COMMON/SCALES/FACSCA,RENSCA
      COMMON/ORDE/NORDER
      COMMON/FLAG/REG1,QUGL1,EXPECT,REG2,GLGL,QUQU,QUQUBS,QUGL2
      LOGICAL REG1,QUGL1,EXPECT,REG2,GLGL,QUQU,QUQUBS,QUGL2
      COMMON/FACREN/FACLM,RENLR,NATFRS
      LOGICAL NATFRS
      COMMON/KOPPAR/LAMBDA,KOPAPP
      COMMON/PVEGAS/NPRINT,IT1,IT2,N1,N2
      DIMENSION IT1(0:20),IT2(0:20),N1(0:20),N2(0:20)
      COMMON/SET/NZP,NWM,NWB,NCONST,NFRS
      COMMON/COLLISION/BEAM,TARGET
C.
C.    ZPROD=.TRUE. : Z production ; ZPROD=.FALSE. : W production
CJS(DATA FILE)      ZPROD=.TRUE.

      IF(NZP.EQ.1)THEN
      ZPROD=.TRUE.
      ELSEIF(NZP.EQ.0)THEN
      ZPROD=.FALSE.
      ENDIF
C.
C.    In case of W production give specifications :
C.    WMINUS=.TRUE. : W(-) production ; WMINUS=.FALSE. : W(+) production
C.    WBOTH=.TRUE.  : W(+) and W(-) production
C.
      IF(NWM.EQ.1)THEN
      WMINUS=.TRUE.
      ELSEIF(NWM.EQ.0)THEN
      WMINUS=.FALSE.
      ENDIF
CJS(DATA FILE)      WMINUS=.TRUE.

      IF(NWB.EQ.1)THEN
      WBOTH=.TRUE.
      ELSEIF(NWB.EQ.0)THEN
      WBOTH=.FALSE.
      ENDIF

CJS(DATA FILE)      WBOTH=.TRUE.
C.
      
C.
C.    Choose a beam particle:
C.    BEAM = 1 : proton
C.           2 : anti-proton
C.           3 : neutron
C.
CJS(DATA FILE)       BEAM=2
C.
C.    Choose a target particle:
C.    TARGET = 1 : proton
C.             2 : neutron
C.
CJS(DATA FILE)      TARGET=1


C.
      S=SQRTS*SQRTS
C.
C.    initialize the scaling variable
C.
      IF (ZPROD) THEN
        TAU=MZ**2/S
      ELSE
        TAU=MW**2/S
      ENDIF
C.
C.    choose the formula for the overall normalization constant
C.    FERMI=.TRUE.   : normalization formula with the Fermi constant
C.    FERMI=.FALSE.  : normalization formula with the fine structure
C.                     constant
C.
      IF(NCONST.EQ.1)THEN
      FERMI=.TRUE.
      ELSEIF(NCONST.EQ.0)THEN
      FERMI=.FALSE.
      ENDIF

CJS(DATA FILE)      FERMI=.TRUE.

C.
C.    Choosing the massfactorization and renormalization scales
C.    NATFRS=.TRUE.  : 'natural' scale = vector boson mass
C.                     FACSCA=RENSCA=SQRT( S*TAU )
C.    NATFRS=.FALSE. : arbitrary scales, should however not be too far
C.                     from the 'natural' scale.
C.                     DO NOT CHANGE THE FORMULAE FOR FACLM AND RENLR
C.

      IF(NFRS.EQ.1)THEN
      NATFRS=.TRUE.
      ELSEIF(NFRS.EQ.0)THEN
      NATFRS=.FALSE.
      ENDIF

C. JS(DATA FILE)    NATFRS=.TRUE.

      IF (NATFRS) THEN
        FACSCA=DSQRT(S*TAU)
        RENSCA=DSQRT(S*TAU)
      ELSE
        FACLM=DLOG(S*TAU/FACSCA**2)
        RENLR=DLOG(S*TAU/RENSCA**2)
      ENDIF

C.
C.    Choose a parton distribution function :
C.    'MSbar mass factorization scheme'
C.
C.    MODE = 1   : CTEQ3M(MS)      : Lambda_4 = 200 MeV
C.           2   : CTEQ3L(LO)      : Lambda_4 = 200 MeV
C.           4   : MRSR1FIT(MS)    : Lambda_4 = 241 MeV
C.           5   : MRSR2FIT(MS)    : Lambda_4 = 344 MeV
C.          10   : GRV92LO(LO)     : Lambda_4 = 200 MeV
C.          11   : GRV92HO(MS)     : Lambda_4 = 200 MeV
C.          12   : GRV94LO(LO)     : Lambda_4 = 200 MeV
C.          13   : GRV94HO(MS)     : Lambda_4 = 200 MeV
C.          15   : GRV98(LO)       : Lambda_4 = 175 MeV
C.          16   : GRV98(MS)       : Lambda_4 = 257 MeV
C.
C.

CJS(DATA FILE)      MODE=12
C.
C.    initialization of the parameters for the coupling constant
C.
C.    LAMBDA   : QCD scale, should match with the scale used in the
C.               parton distribution function chosen above.
C.    KOPAPP   : approximation for the coupling constant
C.               KOPAPP=1  : leading order
C.                     =2  : next-to-leading order
C.
CJS(DATA FILE)      KOPAPP=2
      IF (MODE.EQ.1)  LAMBDA=0.2D0
      IF (MODE.EQ.2)  LAMBDA=0.2D0
      IF (MODE.EQ.4)  LAMBDA=0.241D0
      IF (MODE.EQ.5)  LAMBDA=0.344D0
      IF (MODE.EQ.10) LAMBDA=0.2D0
      IF (MODE.EQ.11) LAMBDA=0.2D0
      IF (MODE.EQ.12) LAMBDA=0.2D0
      IF (MODE.EQ.13) LAMBDA=0.2D0
      IF (MODE.EQ.15) LAMBDA=0.175D0
      IF (MODE.EQ.16) LAMBDA=0.257D0
      IF (MODE.EQ.18) LAMBDA=0.310D0

C
C.
C.    Choose the approximation you want to use for the calculation
C.    of the total cross-section.
C.    NORDER        : order of the q qb contribution ( 0, 1 or 2)
C.    REG1  =.TRUE. : 1st order, q qb, hard is included
C.    QUGL1 =.TRUE. : 1st order, q g, is included
C.    REG2  =.TRUE. : 2nd order, q qb, hard is included
C.    QUGL2 =.TRUE. : 2nd order, q g, is included
C.    GLGL  =.TRUE. : g g contribution is included
C.    QUQU  =.TRUE. : q q and qb qb are included
C.    QUQUBS=.TRUE. : q qb, singlet is included
C.    EXPECT=.TRUE. : in the MSbar scheme : dummy flag
C.    In case you want a complete calculation set NORDER=2 and all the
C.    flags equal to .TRUE.
C.

CJS   NORDER=2

      REG1=.TRUE.
      QUGL1=.TRUE.
      REG2=.TRUE.
      QUGL2=.TRUE.
      GLGL=.TRUE.
      QUQU=.TRUE.
      QUQUBS=.TRUE.
      EXPECT=.TRUE.
C.
C.    VEGAS parameters
C.    ITi(J) and Ni(J) correspond to the integration of function
C     FINTJ in the subroutine INTVEG. Advise: N1 1000 N2 5000
C.
      NPRINT=0
      DO 100 I=0,20
        IT1(I)=5
        N1(I)=1000*4
        IT2(I)=2
        N2(I)=5000*4
100   CONTINUE
      END
      SUBROUTINE DSNEDE(TERM)
C.----------------------------------------------------------------------
C.
C.    This subroutine fills the array TERM(I) with various contributions
C.    to the total cross-section.
C.
C.    All the results will be given in nbarn.
C.
C.    By calling the subroutine INTVEG the folding of the structure
C.    functions PDQQB, PDQG, PDGG, PDQQ and PDQQBS with the correspon-
C.    ding Drell-Yan correction terms is performed.
C.
C.    The results of INTVEG are combined to form the following functions
C.
C.    ZEROTH :   Born cross-section
C.    CONS1  : )
C.    PART1  : ) 1st order, q qb, non-singlet, singular
C.    REST1  : )
C.    HARD1  :   1st order, q qb, non-singlet, regular
C.    QUAGLU :   1st order, q g
C.    CONSCF : )
C.    PARTCF : ) 2nd order, q qb, non-singlet, singular, colour : CF**2
C.    RESTCF : )
C.    CONSCA : )
C.    PARTCA : ) 2nd order, q qb, non-singlet, singular, colour : CA*CF
C.    RESTCA : )
C.    CONSNF : )
C.    PARTNF : ) 2nd order, q qb, non-singlet, singular, colour : nf*CF
C.    RESTNF : )
C.    HARD2  :   2nd order, q qb, non-singlet, regular,
C.    QUAGL2 :   2nd order, qg
C.    GLUGLU :   g g contribution
C.    QUAQUA :   q q and qb qb contributions
C.    QUQUBA :   2nd order, q qb, singlet, contribution.
C.
C.
C.
C.    Some explanation of the jargon used:
C.
C.    In the above we have separated the contributions from the q qb,
C.    non-singlet subprocess into a singular and a regular part.
C.    The singular part is obtained by doing the theoretical calculation
C.    in the soft limit x-->1. The remainder is called the regular part.
C.
C.    The singular part has the following general form :
C.
C.              C*delta(x) + sum_i ( Ci*Di(x) )
C.
C.    the C's are constants, delta(x) is the delta function and Di(x)
C.    is the distribution (log(1-x)**i/(1-x))+.
C.    When one folds such a term with a structure function, one can
C.    divide the result into three parts :
C.    part from the delta function                   ----->  CONS*
C.    two parts from Di(x) :  log(1-tau)**i terms    ----->  PART*
C.                            from VEGAS integration ----->  REST*
C.
C.    For example :
C.    The first order q qb contribution is split into :
C.      1) singular terms :  a)  CONS1
C.                           b)  PART1
C.                           c)  REST1
C.      2) regular terms  :      HARD1
C.
C.
C.
C.    Combining the functions listed above in the right way, one
C.    can obtain the desired approximation for the total cross-section.
C
C.    For example :
C.    In this case we have chosen for the following configuration :
C.
C.    TERM(1)=ZEROTH()                    :  Bron cross-section
C.    TERM(2)=CONS1()+PART1()+REST1()     :  1st order, q qb, singular
C.    TERM(3)=HARD1()                     :  1st order, q qb, regular
C.    TERM(4)=QUAGLU()                    :  1st order, q g
C.    TERM(5)=CONSCF()+PARTCF()+RESTCF()+ :  2nd order, q qb,
C.            CONSCA()+PARTCA()+RESTCA()+ :  non-singlet, singular
C.            CONSNF()+PARTNF()+RESTNF()
C.    TERM(6)=HARD2()                     :  2nd order, q qb,
C.                                        :  non-singlet, regular
C.    TERM(7)=QUAGL2()                    :  2nd order, q g
C.    TERM(8)=GLUGLU()                    :  g g contribution
C.    TERM(9)=QUAQUA()                    :  q q and qb qb contributions
C.    TERM(10)=QUQUBA()                   :  q qb, singlet contribution
C.
C.----------------------------------------------------------------------
      IMPLICIT REAL*8 (A-H,O-Z)
C.
      DIMENSION TERM(20)
C.
      COMMON/ZETA/PI,ZETA2,ZETA3
      COMMON/ENERGY/S,TAU
      COMMON/KOP/ALSQ,ALSQT,FACTOR
      COMMON/CONST/GF,CONVER,ALPHA
      COMMON/ANGLE/SINW,COSW,SINC,COSC
      COMMON/ZORW/ZPROD
      LOGICAL ZPROD
      COMMON/NORM/FERMI
      LOGICAL FERMI
      COMMON/SCALES/FACSCA,RENSCA


C.
C.    subroutine INTVEG folds the parton distribution functions with
C.    the Drell-Yan correction terms.
C.

      CALL INTVEG
C.
C.    FACTOR contains the overall normalization factor for the total
C.    cross-section. The flag FERMI determines whether the formula with
C.    the Fermi constant is taken or the one with the fine structure
C.    constant.
C.
      IF (ZPROD) THEN
        IF (FERMI) THEN
          FACTOR=PI*DSQRT(2.0D0)*GF*TAU/12.0D0
        ELSE
          FACTOR=PI**2*ALPHA/(12.0D0*S*SINW*COSW)
        ENDIF
      ELSE
        IF (FERMI) THEN
          FACTOR=PI*DSQRT(2.0D0)*GF*TAU/3.0D0
        ELSE
          FACTOR=PI**2*ALPHA/(3.0D0*S*SINW)
        ENDIF
      ENDIF
C.

C.    Initialization of the running coupling constant.
C.    COUPL calculates alpha_s
C.    ALSQ  = alpha_s(RENSCA) / (4*pi)
C.    ALSQT = alpha_s(S*TAU*(1-TAU)) / (4*pi)
C.    ALSQT is only used in the DIS scheme for exponentiation
C.
      ALSQ=COUPL(RENSCA)/4.0D0/PI
      PP=4.0D0*PI*ALSQ
CJS      PRINT*,'RENSCA=',RENSCA
CJS      PRINT*,'ALSQ=',ALSQ
CJS      PRINT*,'PP=',PP
      
      ALSQT=COUPL(S*TAU*(1.0D0-TAU))/4.0D0/PI
C.
      DO 100 I=1,20
        TERM(I)=0.0D0
100   CONTINUE
C.
C.    The array TERM(i) should be filled with the various contributions
C.    to the total cross-section.
C.
      TERM(1)=ZEROTH()
      TERM(2)=CONS1()+PART1()+REST1()
      TERM(3)=HARD1()
      TERM(4)=QUAGLU()
      TERM(5)=CONSCF()+PARTCF()+RESTCF()+
     .        CONSCA()+PARTCA()+RESTCA()+
     .        CONSNF()+PARTNF()+RESTNF()
      TERM(6)=HARD2()
      TERM(7)=QUAGL2()
      TERM(8)=GLUGLU()
      TERM(9)=QUAQUA()
      TERM(10)=QUQUBA()

C.
C.    conversion to nbarn
C.
      DO 10 I=1,20
        TERM(I)=CONVER*TERM(I)
10    CONTINUE
      END
